diff --git a/src/core/ngx_connection.h b/src/core/ngx_connection.h
index f9bf5040..2f41e74 100644
--- a/src/core/ngx_connection.h
+++ b/src/core/ngx_connection.h
@@ -154,6 +154,7 @@ struct ngx_connection_s {
     unsigned            idle:1;
     unsigned            reusable:1;
     unsigned            close:1;
+    unsigned            keepalive:1;

     unsigned            sendfile:1;
     unsigned            sndlowat:1;
diff --git a/src/http/ngx_http_upstream.c b/src/http/ngx_http_upstream.c
index dc3babd..0a7b521 100644
--- a/src/http/ngx_http_upstream.c
+++ b/src/http/ngx_http_upstream.c
@@ -85,6 +85,9 @@ static ngx_int_t ngx_http_upstream_process_expires(ngx_http_request_t *r,
     ngx_table_elt_t *h, ngx_uint_t offset);
 static ngx_int_t ngx_http_upstream_process_accel_expires(ngx_http_request_t *r,
     ngx_table_elt_t *h, ngx_uint_t offset);
+static ngx_int_t ngx_http_upstream_process_connection(ngx_http_request_t *r,
+                                                      ngx_table_elt_t *h,
+                                                      ngx_uint_t offset);
 static ngx_int_t ngx_http_upstream_process_limit_rate(ngx_http_request_t *r,
     ngx_table_elt_t *h, ngx_uint_t offset);
 static ngx_int_t ngx_http_upstream_process_buffering(ngx_http_request_t *r,
@@ -220,7 +223,7 @@ ngx_http_upstream_header_t  ngx_http_upstream_headers_in[] = {
                  offsetof(ngx_http_headers_out_t, accept_ranges), 1 },

     { ngx_string("Connection"),
-                 ngx_http_upstream_ignore_header_line, 0,
+                 ngx_http_upstream_process_connection, 0,
                  ngx_http_upstream_ignore_header_line, 0, 0 },

     { ngx_string("Keep-Alive"),
@@ -3378,6 +3381,21 @@ ngx_http_upstream_process_accel_expires(ngx_http_request_t *r,


 static ngx_int_t
+ngx_http_upstream_process_connection(ngx_http_request_t *r,
+                                     ngx_table_elt_t *h,
+                                     ngx_uint_t offset) {
+
+    if (ngx_strncasecmp(h->value.data, (u_char *) "close", h->value.len)
+        != 0)
+    {
+        r->upstream->headers_in.keepalive = 1;
+    }
+
+    return NGX_OK;
+}
+
+
+static ngx_int_t
 ngx_http_upstream_process_limit_rate(ngx_http_request_t *r, ngx_table_elt_t *h,
     ngx_uint_t offset)
 {
diff --git a/src/http/ngx_http_upstream.h b/src/http/ngx_http_upstream.h
index c915b26..8539c62 100644
--- a/src/http/ngx_http_upstream.h
+++ b/src/http/ngx_http_upstream.h
@@ -263,6 +263,7 @@ typedef struct {
     ngx_array_t                      cache_control;

     unsigned                         chunked:1;
+    unsigned                         keepalive:1;
 } ngx_http_upstream_headers_in_t;

